#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <string>
#include <exception>
#include <vector>

using namespace std;

#ifdef WIN32
#include <Winsock2.h>
#define snprintf _snprintf
#else
#define closesocket close
#define SOCKET int
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#endif

#include "zival.h"

// -------[ socket ]------------------------------------

class TSock {
private:
	SOCKET sock;
	string buffer;
public:
	TSock();
	~TSock();

	bool Connect(char* hostname, int port);
	string readline();
	vector<string> readlineV();
	void writeline(string str);
};

TSock::TSock() : sock(-1), buffer() {}
TSock::~TSock() { closesocket(sock); }

bool TSock::Connect(char *hostname, int port) {
	sockaddr_in saddr;
	struct hostent *he;

#ifdef WIN32
	WORD wVersionRequested = MAKEWORD( 2, 2 );
	WSADATA wsaData;
	WSAStartup(wVersionRequested, &wsaData);
#endif

	he = gethostbyname(hostname);
	if (!he) return false;

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);
	saddr.sin_addr.s_addr = (*(struct in_addr*)he->h_addr).s_addr;
	
	sock = socket(PF_INET, SOCK_STREAM, 0);
	if (sock == -1) return false;
	if (connect(sock, (const sockaddr*)&saddr, sizeof(saddr)) != 0) return false;
	return true;
}

void chop(string &s) {
	if (s[s.length()-1] == '\n') s.erase(s.length()-1);
	if (s[s.length()-1] == '\r') s.erase(s.length()-1);
}

string TSock::readline() {
	string::size_type pos;
	while (true) {
		if ((pos=buffer.find('\n')) != string::npos) {
			string q = buffer.substr(0, pos);
			buffer.erase(0, pos+1);
			chop(q);
			return q;
		} else {
			char tmp[130];
			int s = recv(sock, tmp, 128, 0);
			if (s < 0) throw new string("Napaka pri komunikaciji s streznikom");
			if (s == 0) throw new string("Streznik je zaprl povezavo");
			tmp[s] = '\0';
			buffer += string(tmp);
		}
	}
}

vector<string> TSock::readlineV() {
	string l = readline();
	vector<string> v;
	
	string::size_type fpos, lpos = 0;
	while (lpos < l.length()) {
		fpos = l.find(' ', lpos);
		if (fpos == string::npos) fpos = l.length();
		string x = l.substr(lpos, fpos-lpos);
		v.push_back(x);
		lpos = fpos+1;
	}

	return v;
}
void TSock::writeline(string str) {
	send(sock, str.c_str(), (int)str.length(), 0);
}

// -----------------------------------------------------

void connect(char *hostname, TSock &sock) {
	if (!sock.Connect(hostname, 1174)) throw new string("Povezava s streznikom neuspesna");

	string server = sock.readline();
	printf("%s\n", server.c_str());

	sock.writeline(string("AVE ") + (Zival::Tip == pasmaMuca ? "MACKA " : "MIS ") + Zival::Ime + "\n");
	string succ = sock.readline();

	if (succ != "PRIJAVA USPESNA") throw new string((string("Prijava na streznik neuspesna: ") + succ).c_str());
}

void event_loop(TSock &sock) {
	while (true) {
		vector<string> cmda = sock.readlineV();
		if (cmda[0] == "ADIJO") return;
		if (cmda[0] == "KONEC_IGRE") continue;
		if (cmda[0] != "ZBUDI_SE") throw new string("Napaka pri komunikaciji s streznikom");
		
		Koordinate j(atof(cmda[1].c_str()), atof(cmda[2].c_str())), n(atof(cmda[3].c_str()), atof(cmda[4].c_str()));
		Zival *z = new Zival();

		double ZeljenaSmer;
		z->ZbudiSe(j, n, ZeljenaSmer);

		char tmp[100];
		snprintf(tmp, 100, "BUDNA %f\n", ZeljenaSmer);
		sock.writeline(tmp);

		while (true) {
			cmda = sock.readlineV();

			if (cmda[0] == "ADIJO") return;
			if (cmda[0] == "KONEC_IGRE") {
				z->KonecIgre(razlog(atoi(cmda[1].c_str())));
				delete z;
				break;
			} else if (cmda[0] == "QV") {
				double SpremembaHitrosti, SpremembaSmeri;
				char tmp[256];
				Stanje sj(atof(cmda[1].c_str()), atof(cmda[2].c_str()), atof(cmda[3].c_str()), atof(cmda[4].c_str()));
				Stanje sn(atof(cmda[5].c_str()), atof(cmda[6].c_str()), atof(cmda[7].c_str()), atof(cmda[8].c_str()));

				z->KamGres(sj, sn, SpremembaSmeri, SpremembaHitrosti);
				snprintf(tmp, 256, "QVR %f %f\n", SpremembaSmeri, SpremembaHitrosti);
				sock.writeline(tmp);
			}
		}
	}
}

int main(int argc, char* argv[]) {
	char *hostname;
	TSock sock;

	if (argc < 2) hostname = "localhost";
	else hostname = argv[1];

	try {
		connect(hostname, sock);
		event_loop(sock);
	} catch (string *e) {
		printf("%s\n", e->c_str());
	}

	return 0;
}
